/**
 * This file is part of the Detox package.
 *
 * Copyright (c) Doug Harple <detox.dharple@gmail.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

#ifndef __CONFIG_FILE_DUMP_H
#define __CONFIG_FILE_DUMP_H

#include "detox_struct.h"

extern void dump_config_file(config_file_t *config_file, options_t *main_options);

#endif /* __CONFIG_FILE_DUMP_H */
