/**
 * This file is part of the Detox package.
 *
 * Copyright (c) Doug Harple <detox.dharple@gmail.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

#ifndef __TABLE_DUMP_H
#define __TABLE_DUMP_H

#include "detox_struct.h"

extern void table_dump(table_t *table, int verbose);
extern void table_stats(table_t *table);

#endif /*  __TABLE_DUMP_H */

