/**
 * This file is part of the Detox package.
 *
 * Copyright (c) Doug Harple <detox.dharple@gmail.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 */

#ifndef __PARSE_TABLE_H
#define __PARSE_TABLE_H

#include "detox_struct.h"

extern table_t *parse_table(char *filename);

#endif /* __PARSE_TABLE_H */
